import 'dart:ffi';

import 'package:flutter/material.dart';
import 'package:csiapi/Helper/color.dart';
import 'package:csiapi/constant/assets.dart';

class DefaultTextField extends StatefulWidget {

  final String hint;
  final String label;
  final String errorMessage;
  final String obscuringCharacter;
  final bool enabled;
  final Function onTap;
  // final Function onChange;
  final TextEditingController controller;
  final TextInputType keyboardType;
  final EdgeInsets margin;
  final bool obscureText;
  final TextInputAction action;
  // final TextInputType textInputType;
  final TextInputAction textInputAction;
  final Function(String) onFieldSubmitted;
  final Function(String) onChanged;
  final Widget suffix;
  final FocusNode focusNode;
  final int maxLines;
  final int minLines;
  final String assets;
  final FocusNode node;
  final BorderRadius borderRadius;
  final Border border;
  final bool enableTextView;
  final Color colorsText;
  final bool isPassword;
  final String suffixIcon;
  final Icon suffixIcons;
  final bool isSearch;
  final Color colorHint;
  final String customSuffixIcon;
  final Function onClear;

  const DefaultTextField(
      {Key key,
        this.hint,
        this.onClear,
        this.label,
        this.errorMessage,
        this.obscuringCharacter = '●',
        this.enabled = true,
        this.onTap,
        this.action,
        this.keyboardType,
        this.maxLines = 1,
        this.minLines = 1,
        @required this.controller,
        this.margin = const EdgeInsets.all(0),
        this.obscureText,
        // this.textInputType = TextInputType.text,
        this.textInputAction = TextInputAction.done,
        this.onFieldSubmitted,
        this.onChanged,
        // this.onChange,
        this.suffix,
        this.assets,
        this.node,
        this.borderRadius,
        this.border,
        this.colorsText,
        this.enableTextView,
        this.colorHint,
        this.isPassword,
        this.suffixIcon,
        this.suffixIcons,
        this.customSuffixIcon,
        this.isSearch,
        this.focusNode})
      : super(key: key);

  _defaultTextField createState() => _defaultTextField();

}

class _defaultTextField extends State<DefaultTextField> {

  bool isHide;

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return Container(
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
        mainAxisSize: MainAxisSize.max,
        children: [
          widget.customSuffixIcon == null ? Container() :
          Container(
            alignment: Alignment.topLeft,
            margin: EdgeInsets.only(left: 8,right: 8),
            height: 30,
            width: 30,
            child: Image.asset(widget.customSuffixIcon),
          ),
          widget.suffixIcons == null ? Container() : Container(
            height: 20,
            width: 20,
            alignment: Alignment.center,
            // color: Colors.red,
            // margin: EdgeInsets.all(8),
            child: widget.suffixIcons,
          ),
          Expanded(
            child: TextField(
              controller: widget.controller,
              onChanged: widget.onChanged == null ? (s){
                setState(() {

                });
              } : widget.onChanged,
              onTap: widget.onTap,
              obscureText: isHide,
              autocorrect: false,
              textInputAction: widget.textInputAction,
              // onFieldSubmitted: widget.onFieldSubmitted,
              maxLines: widget.maxLines == null ? 1 : widget.maxLines,
              keyboardType: widget.keyboardType,
              style: TextStyle(
                  color: widget.colorsText,
                  // fontFamily: Fonts.avenir
              ),
              enabled: widget.enableTextView == null ? true : widget.enableTextView,
              decoration: InputDecoration(
                  isDense: true,
                  contentPadding: EdgeInsets.only(bottom: 4,top: 4),
                  hintText: widget.hint,
                  border: InputBorder.none,
                  focusedBorder: InputBorder.none,
                  enabledBorder: InputBorder.none,
                  errorBorder: InputBorder.none,
                  disabledBorder: InputBorder.none,
                  labelStyle: TextStyle(
                    // fontFamily: Fonts.avenir,
                  ),
                  hintStyle: TextStyle(
                      // fontFamily: Fonts.avenir,
                      color: widget.colorHint == null ? null : widget.colorHint
                  ),
                  suffixIcon: widget.isSearch != null ?
                  IconButton(
                    icon: Icon(Icons.search,color: ColorsState.colorPrimaryBlue,),
                    // Image.asset(widget.suffixIcon,scale: 1,),
                  )
                      : null,
                  prefixIcon: widget.assets == null ? null : IconButton(
                    alignment: Alignment.topLeft,
                    onPressed: (){

                    },
                    icon: Image.asset(widget.assets,scale: 1,alignment: Alignment.topLeft,),
                  )
              ),

            ),
          ),
          widget.suffixIcon == null ? Container() :
          Container(
            margin: EdgeInsets.only(top: 13,bottom: 10),
            alignment: Alignment.center,
            height: 20,
            width: 20,
            child: Image.asset(widget.suffixIcon),
          ),
          widget.controller != null ? widget.controller.text.isNotEmpty ?
          GestureDetector(
            onTap:widget.onClear == null ? (){
              widget.controller.text = '';
              setState(() {

              });
            } : widget.onClear ,
            child: Container(
                child: Icon(Icons.close)),
          ) : Container() : Container(),
          widget.isPassword != null ?
          GestureDetector(
            onTap: (){
              if(isHide){
                isHide = false;
                setState(() {
                });
              }else{
                isHide = true;
                setState(() {
                });
              }

            },
            child: Container(
              width: 25,
              height: 25,
              alignment: Alignment.center,
              child: Image.asset(isHide ? Assets.icon_eye :Assets.icon_eye_in),
              // child: Icon(Icons.open),
            ),
          )
              :
          Container()
        ],
      ),
    );

  }

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    // if(widget.isPassword == null){
    //   isHide = false;
    // }else{

    if(widget.isPassword != null){
      isHide = true;
    }else{
      if (widget.obscureText == null) {
        isHide = false;
      }else{
        isHide = widget.obscureText;
      }
    }
  }
// }
}


