import 'package:flushbar/flushbar.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:csiapi/Helper/color.dart';
import 'package:csiapi/Helper/defaultText.dart';
import 'package:csiapi/network/config.dart';
import 'package:csiapi/preferences/preferences.dart';
import 'package:csiapi/route/myRouter.dart';
import 'package:shimmer/shimmer.dart';

class Ui {

  BuildContext context;

  Ui(this.context);

  static Ui of(BuildContext context) {
    return Ui(context);
  }

  static Widget shimmer({
    double height = 48,
    double width = 48,
    double borderRadius = 0,
    Color baseColor,
    Color highlightColor,
    EdgeInsets margin = const EdgeInsets.all(0),
    ShimmerDirection direction = ShimmerDirection.ltr,
  }){
    return Padding(
      padding: margin,
      child: Shimmer.fromColors(
        baseColor: baseColor ?? Colors.grey[200],
        highlightColor: highlightColor ?? Colors.grey[300],
        direction: direction,
        child: Container(
          decoration: BoxDecoration(
              color: Colors.grey,
              borderRadius: BorderRadius.all(Radius.circular(borderRadius))
          ),
          height: height,
          width: width,
        ),
      ),
    );
  }

  successAlert({String title = '', String message = '', int duration = 3}){
    _alert(title: title, message: message, duration: duration);
  }

  errorAlert({String title = '', String message = '', int duration = 3}){
    _alert(title: title, message: message, duration: duration, isSuccess: false);
  }

  _alert({String title = '', String message = '', int duration = 3, bool isSuccess = true}){
    Flushbar(
      title: title,
      messageText: Text(message,
        maxLines: 10,
        style: TextStyle(
            color: Colors.white
        ),
      ),
      duration: Duration(seconds: duration),
      backgroundColor: isSuccess ? Colors.orange : Color(0xFF303030),
      flushbarPosition: isSuccess ? FlushbarPosition.TOP : FlushbarPosition.BOTTOM,
    )..show(context);
  }

  alertPopUp({String message})async{
    showDialog(
        context: context,
        builder: (_) => new AlertDialog(
          backgroundColor: Config.colorMain,
          content: SingleChildScrollView(
            scrollDirection: Axis.vertical,
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                Container(
                    alignment: Alignment.center,
                    width: 300,
                    decoration: BoxDecoration(
                      borderRadius: BorderRadius.only(topLeft: Radius.circular(5),topRight: Radius.circular(5)),
                      color: Colors.white,
                    ),
                    padding: EdgeInsets.all(25),
                    child: Text(message == null ? '' : message)),
                GestureDetector(
                  onTap: (){
                    myRouter.pop(context);
                  },
                  child: DefaultText(
                    padding: EdgeInsets.all(20),
                    borderRadius: BorderRadius.only(bottomLeft: Radius.circular(5),bottomRight: Radius.circular(5)),
                    alignment: Alignment.center,
                    width: 300,
                    textLabel: "Ok",
                    colorsText: Colors.white,
                  ),
                )
              ],
            ),
          ),
          contentPadding: EdgeInsets.all(0),
          insetPadding: EdgeInsets.all(20),
        ));


  }

  dialogPopUp({String title,String buttonText,String content,Function onTap,Function onCancel,bool isSuccess}){
    Color color = isSuccess ? Colors.greenAccent : ColorsState.colorPrimaryRed;
    showDialog(
        context: context,
        builder: (_) => new AlertDialog(
          backgroundColor: Colors.white,
          content: SingleChildScrollView(
            scrollDirection: Axis.vertical,
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                // Container(
                //   height:60,
                //   width: 60,
                //   margin: EdgeInsets.only(top: 40),
                //   child: Image.asset(isSuccess ? Assets.successIcon : Assets.alertIcon),
                // ),
                DefaultText(
                 textLabel: title,
                 margin: EdgeInsets.only(top: 8),
                 alignment: Alignment.center,
                 width: MediaQuery.of(context).size.width,
                  colorsText: color,
                  fontWeight: FontWeight.bold,
                  sizeText: 16,
                ),
                DefaultText(
                  textLabel: content,
                  margin: EdgeInsets.only(top: 8),
                  alignment: Alignment.center,
                  width: MediaQuery.of(context).size.width,
                  colorsText: Colors.grey,
                  sizeText: 14,
                ),

                onCancel != null ?
                Row(
                  mainAxisSize: MainAxisSize.max,
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [

                    GestureDetector(
                      onTap: onTap,
                      child: DefaultText(
                        borderRadius: BorderRadius.all(Radius.circular(10)),
                        margin: EdgeInsets.all(20),
                        textLabel: buttonText == null ? 'Yes' : buttonText,
                        colorsText: Colors.white,
                        colorbackground: Colors.red,
                        padding: EdgeInsets.only(left: 30,right: 30,top: 8,bottom: 8),

                      ),
                    ),

                    // Container(
                    //   width: 30,
                    // ),
                    GestureDetector(
                      onTap: onCancel,
                      child: DefaultText(
                        borderRadius: BorderRadius.all(Radius.circular(10)),
                        margin: EdgeInsets.all( 20),
                        textLabel:  'Cancel',
                        colorsText: ColorsState.colorPrimaryRed,
                        colorbackground: Colors.white,
                        border: Border.all(color: Colors.red,width: 1),
                        padding: EdgeInsets.only(left: 16,right: 16,top: 8,bottom: 8),

                      ),
                    ),
                    // Container(
                    //   margin: EdgeInsets.only(bottom: 20),
                    //   child: RaisedButton(onPressed: onCancel,
                    //     color: color,
                    //     child: DefaultText(
                    //       textLabel:  'Cancel',
                    //       colorsText: Colors.white,
                    //     ),
                    //   ),
                    // ),

                    // Container(
                    //   margin: EdgeInsets.only(bottom: 20),
                    //   child: RaisedButton(onPressed: onTap == null ? (){
                    //     myRouter.pop(context);
                    //   } : onTap ,
                    //     color: color,
                    //     child: DefaultText(
                    //       textLabel: buttonText == null ? 'Ok' : buttonText,
                    //       colorsText: Colors.white,
                    //     ),
                    //   ),
                    // )
                  ],
                ) :
                Container(
                  margin: EdgeInsets.all(20),
                  width: MediaQuery.of(context).size.width,
                  child: RaisedButton(onPressed: onTap == null ? (){
                    myRouter.pop(context);
                  } : onTap ,
                    color: color,
                    child: DefaultText(
                      textLabel: buttonText == null ? 'Ok' : buttonText,
                      colorsText: Colors.white,
                    ),
                  ),
                ),

                // Container(
                //     alignment: Alignment.center,
                //     width: MediaQuery.of(context).size.width,
                //     decoration: BoxDecoration(
                //       borderRadius: BorderRadius.only(topLeft: Radius.circular(5),topRight: Radius.circular(5)),
                //       color: ColorsState.white,
                //     ),
                //     padding: EdgeInsets.all(25),
                //     child: Text(message)),

                // Row(
                //   children: [
                //     Expanded(
                //       child: GestureDetector(
                //         onTap: onOke,
                //         child: DefaultText(
                //           padding: EdgeInsets.all(20),
                //           border: Border.all(color: !isNext ? Colors.black  : ColorsState.colorPrimaryBlue,width: 0.5) ,
                //           borderRadius: BorderRadius.only(bottomLeft: Radius.circular(5)),
                //           alignment: Alignment.center,
                //           width: MediaQuery.of(context).size.width,
                //           textLabel: "Yes",
                //           colorsText: isNext ? Colors.white : Colors.black,
                //           colorbackground: isNext ? ColorsState.colorPrimaryBlue : Colors.white,
                //         ),
                //       ),
                //     ),
                //     Expanded(
                //       child: GestureDetector(
                //         onTap: onCancel,
                //         child: DefaultText(
                //           padding: EdgeInsets.all(20),
                //           border:  Border.all(color: isNext ? Colors.black : Colors.red,width: 0.5) ,
                //           borderRadius: BorderRadius.only(bottomRight: Radius.circular(5)),
                //           alignment: Alignment.center,
                //           width: MediaQuery.of(context).size.width,
                //           textLabel: "No",
                //           colorsText: !isNext ? Colors.black : Colors.black,
                //           colorbackground: !isNext ? Colors.red : Colors.white,
                //         ),
                //       ),
                //     ),
                //   ],
                // )
              ],
            ),
          ),
          contentPadding: EdgeInsets.all(0),
          insetPadding: EdgeInsets.all(20),
        ));
  }



}

